@@ -291,7 +291,7 @@ void spc_allow(struct burn_drive *d)
 }
 
 /*
-ts A70518 : Do not call with *alloc_len < 8
+ts A70518 - A90603 : Do not call with *alloc_len < 10
 */
 /** flag&1= do only inquire alloc_len */
 static int spc_sense_caps_al(struct burn_drive *d, int *alloc_len, int flag)
@@ -303,13 +303,19 @@ static int spc_sense_caps_al(struct burn_drive *d, int *alloc_len, int flag)
 	unsigned char *page;
 	struct command c;
 	struct burn_speed_descriptor *sd;
+	char msg[BURN_DRIVE_ADR_LEN + 160];
 
 	/* ts A61225 : 1 = report about post-MMC-1 speed descriptors */
 	static int speed_debug = 0;
 
-	if (*alloc_len < 8)
+	if (*alloc_len < 10)
 		return 0;
 
+	/* ts A90602 : Clearing mdata before command execution */
+	m = d->mdata;
+	m->valid = 0;
+	burn_mdata_free_subs(m);
+
 	memset(&buf, 0, sizeof(buf));
 	scsi_init_command(&c, SPC_MODE_SENSE, sizeof(SPC_MODE_SENSE));
 /*
@@ -328,12 +334,11 @@ static int spc_sense_caps_al(struct burn_drive *d, int *alloc_len, int flag)
 	d->issue_command(d, &c);
 	if (c.error) {
 		memset(&buf, 0, sizeof(buf));
-		d->mdata->valid = -1;
+		m->valid = -1;
 		was_error = 1;
 	}
 
 	size = c.page->data[0] * 256 + c.page->data[1];
-	m = d->mdata;
 	page = c.page->data + 8;
 
 	/* ts A61225 :
@@ -342,18 +347,28 @@ static int spc_sense_caps_al(struct burn_drive *d, int *alloc_len, int flag)
 	   in MMC-3 6.3.11 there are at least 28 bytes plus a variable length
 	   set of speed descriptors. In MMC-5 E.11 it is declared "legacy".
 	*/
+	/* ts A90603 :
+	   SPC-1 8.3.3 enumerates mode page format bytes from 0 to n and
+	   defines Page Length as (n-1).
+	*/
 	page_length = page[1];
 	old_alloc_len = *alloc_len;
-	*alloc_len = page_length + 8;
+	*alloc_len = page_length + 10;
 	if (flag & 1)
 		return !was_error;
-	if (page_length + 8 > old_alloc_len)
-		page_length = old_alloc_len - 8;
-	if (page_length < 22)
-		return 0;
+	if (page_length + 10 > old_alloc_len)
+		page_length = old_alloc_len - 10;
 
-	m->valid = 0;
-	burn_mdata_free_subs(m);
+	/* ts A90602 : 20 asserts page[21]. (see SPC-1 8.3.3) */
+	if (page_length < 20) {
+		m->valid = -1;
+		sprintf(msg, "MODE SENSE page 2A too short: %s : %d",
+			d->devname, page_length);
+		libdax_msgs_submit(libdax_messenger, d->global_index,
+				0x0002016e, LIBDAX_MSGS_SEV_DEBUG,
+				LIBDAX_MSGS_PRIO_LOW, msg, 0, 0);
+		return 0;
+	}
 
 	m->buffer_size = page[12] * 256 + page[13];
 	m->dvdram_read = page[2] & 32;
@@ -384,12 +399,13 @@ static int spc_sense_caps_al(struct burn_drive *d, int *alloc_len, int flag)
 	m->min_end_lba = 0x7fffffff;
 	m->max_end_lba = 0;
 
-	m->valid = 1;
+	if (!was_error)
+		m->valid = 1;
 
 	mmc_get_configuration(d);
 
 	/* ts A61225 : end of MMC-1 , begin of MMC-3 */
-	if (page_length < 32) /* no write speed descriptors ? */
+	if (page_length < 30) /* no write speed descriptors ? */
 		goto try_mmc_get_performance;
 
 	m->cur_write_speed = page[28] * 256 + page[29];
@@ -413,12 +429,12 @@ static int spc_sense_caps_al(struct burn_drive *d, int *alloc_len, int flag)
 	}
 
 	for (i = 0; i < num_write_speeds; i++) {
-		speed = page[32 + 4*i + 2] * 256 + page[32 + 4*i + 3];
+		speed = page[32 + 4 * i + 2] * 256 + page[32 + 4 * i + 3];
 
 		if (speed_debug) 
 		fprintf(stderr,
 			"LIBBURN_DEBUG: write speed #%d = %d kB/s  (rc %d)\n",
-			i, speed, page[32 + 4*i +1] & 7);
+			i, speed, page[32 + 4 * i + 1] & 7);
 
 		/* ts A61226 */
 		ret = burn_speed_descriptor_new(&(d->mdata->speed_descriptors),
@@ -431,7 +447,7 @@ static int spc_sense_caps_al(struct burn_drive *d, int *alloc_len, int flag)
 				strcpy(sd->profile_name,
 					d->current_profile_text);
 			}
-			sd->wrc = (( page[32 + 4*i +1] & 7 ) == 1 );
+			sd->wrc = (( page[32 + 4 * i + 1] & 7 ) == 1 );
 			sd->write_speed = speed;
 		}
 
@@ -459,7 +475,7 @@ try_mmc_get_performance:;
 
 void spc_sense_caps(struct burn_drive *d)
 {
-	int alloc_len, start_len = 22, ret;
+	int alloc_len, start_len = 30, ret;
 
 	if (mmc_function_spy(d, "sense_caps") <= 0)
 		return;
